import datetime
from re import A
import matplotlib.pyplot as plt

import iexcode.instruments.cfg as iex
from iexcode.instruments.ARPES import ARPES_init
from iexcode.instrumetns.kappa import kappa_init

from iexcode.instruments.scanRecord import *
from iexcode.instruments.diagnostics import _diagnostics_dict, diagnostics_all_out, diagnostics_all_in,diode_c,diode_d
from iexcode.instruments.current_amplifiers import *
from iexcode.instruments.xrays import *
from iexcode.instruments.VLS_PGM import *
from iexcode.instruments.Logfile import log_name_set,log_print
from iexcode.instruments.shutters import branch_shutter_open, branch_shutter_status
from iexcode.instruments.utilities import *
from iexcode.macros.commissioning import *
from iexcode.macros.quick_plot import *


def detectors(branch):
    """
    returns  detCA4,detH,detV,detDiode
    """
    DetDict={'c':(9,7,8,15),'d':(9,7,8,14)}
    return DetDict[branch]



def interactive_fct(mirror=False): 
    options='0,1,2,3'
    if not mirror:
        print('''\nDo you want to:
        \t (0) write comment
        \t (1) continue
        \t (2) repeat scan
        \t (3) adjust xrange for fit
        ''')
    else:
        print('''\nDo you want to:
        \t (0) write comment
        \t (1) continue
        \t (2) repeat scan
        \t (3) adjust xrange for fit
        \t (4) move M1 roll
        ''')
        options=options+',4'
    flag=input('\nType ('+options+') >')
    if flag not in options.split(","):
        print('\ninvalid input; please try again...\n')
        flag=input('\nType ('+options+') >')
    if flag == '0': 
        cmt=input('\nType comment >')
        print('\n================================')
        log_print(cmt)
        print('================================\n')
        flag=input('\nType ('+options+') >')
    if flag == '1': print('\nresuming script...\n'); flag=1
    if flag == '2': print('\nrepeating scan...\n'); flag=2
    if flag == '3': flag=3
    if mirror:
        if flag == '4': flag=4
    
    return flag
   

def StartOfTheWeek(GRT,branch,wait=False,**kwargs):
    """
    This should be run every at the start of every week; 50 min total.
    Switch to C-branch,.
    If wait=True, wait for next day a 8:05.
    
    **kwargs:
        mode: polarization, default => RCP
        repeat: True/False
            True => doesn't do any initialization 
            False => resets everything (switches branch and grating)
        scanType = ['slit1','wire','monoVslit','flux',]
        sound: plays sound when a scantype is completed, default => True
        # for individiual scantypes
        slit1A_extended: to take a larger range, default => False
        monoVslit_quick: 
            True => takes a single line cut (defaule)
            False => 
        
       """
    kwargs.setdefault('mode','RCP')
    kwargs.setdefault('repeat',False)
    kwargs.setdefault('scanType',['slit1','wire','flux','monoVslit'])
    kwargs.setdefault('sound',True)
       
    kwargs.setdefault('slit1A_extended',False)
    kwargs.setdefault('monoVslit_quick',True)


    detCA4,detH,detV,detDiode = detectors()

    branch=branch.lower()
    if iex.BL == None:
        if branch == 'c':
            BL_init=ARPES_init
        else:
            BL_init=kappa_init

    if kwargs['repeat']==False: #reset folders/detectors/...
        BL_init('staff')
    else:
        BL_init('staff',reset=False,set_folders=False)
        switch_branch(branch)
        log_name_set('StartOfTheWeek_log.txt')
        log_print("\n\n================== Start Of The Week @ "+today('slash')+':\n',FileName=FileName)
         
    for scan in kwargs['scanType']: 
        if scan not in ['slit1','wire','flux','monoVslit']:
            print(scan+" is not a valid scan scanType=['slit1','wire','flux','monoVslit']")
            return
       
    ### checking diode settings:
    if branch == 'c': 
        ca15 = Keithley('b',15)
        ca15.autoscale('Off')

    else:
        foo=input('Do you have a diode in direct beam (y or n)? >')
        if foo.lower() == 'y' or foo.lower() == 'yes':
            print('Resuming...')
    
    print("\n\n================== Sets Beamline & Scans:")
    grating(GRT)

    ###########################################################################
    ###        stuff that requires beam
    ###########################################################################

    ### checking branch shutter:
    shutter_open = branch_shutter_status()
    if shutter_open == False:
        foo=input_d('Shutter '+branch.upper()+' is closed, do you want to open it (y or n)? >')
        if foo.lower() == 'y' or foo.lower() ==  'yes':
            branch_shutter_open()
        else:
            print('Aborting...')
            return

    ### checking QP ratio:
    QP=caget('ID29:QuasiRatio.RVAL')
    if QP != 100:
        foo=input('QP on!!! Continue (y or n)? >')
        if foo.lower() == 'y' or foo.lower() == 'yes':
            print('Resuming...')

    ### Wait for next 8AM:
    if wait:
        t = datetime.today()
        if 0 <= t.hour <= 8:
            wait_for_it(0,8,5)
        else:    
            wait_for_it(1,8,5)

    ### checking ID:
    polarization(kwargs['mode'])
            
    ### Ready to start:
    FirstScan=iex.BL.mda.filenum()

    ###########################################################################
    if 'slit1' in kwargs['scanType']:
        print("\n\n================== Slit 1A scans:")
        mvID(2000)
        flag=2
        while flag==2:
            #scanning
            slit1A_kwargs = {
                'sound':kwargs['sound'],
                'extended_range':kwargs['slit1A_extended']
                }
            ID_steering_scans(**slit1A_kwargs)

            #plotting
            try:
                scanNum_h = last_mda()-1
                scanNum_v = last_mda()
                h_position = ID_steering_plots(scanNum_h,detCA4,xrange=[-1,1])
                v_position = ID_steering_plots(scanNum_v,detCA4,xrange=[-1,1])
                iex_steering(h_position,v_position)
            except:
                print('\nUnable to fit position; try to adjust xrange.\n')

            flag=interactive_fct()
            
            #want to refit
            while flag == 3:
                try:
                    fitH=input('do you want to fit mda #'+str(m-1)+' ie horizontal (y or n)? >')
                    if fitH.lower() == 'y' or fitH.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        h_position = ID_steering_plots(scanNum_h,detCA4,xrange=[float(x1),float(x2)])                    fitV=input('do you want to fit mda #'+str(m)+' ie vertical (y or n)? >')
                    
                    if fitV.lower() == 'y' or fitV.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        v_position = ID_steering_plots(scanNum_v,detCA4,xrange=[float(x1),float(x2)])

                    iex_steering(h_position,v_position)
                    foo=input('\nare you happy with the fit (y or n)? >')
                    if foo.lower() == 'y' or foo.lower() ==  'yes': 
                        flag=interactive_fct()
                    

                except:
                    print('\nUnable to fit position; try to tweak xrange again\n') 
                    flag=interactive_fct()

    ###### Wire scans: 
    if 'wire' in kwargs['scanType']:
        #scanning
        mvID(2000)
        wire_scans(**kwargs)
        
        #plotting
        try:
            scanNum_h = last_mda()-1
            scanNum_v = last_mda()
            plot_mda(scanNum_h,detH,scanNum_v,detV,title='wire-H (blue) & wire-V (orange)');plt.show()
        except:
            print('Unable to plot. Try:')
            print("plot_mda("+str(scanNum_h)+","+str(detH)+","+str(scanNum_v)+","+str(detV)+",title='wire-H (blue) & wire-V (orange)');plt.show()")
              

    ###### Mono/slit scans: 
    if 'monoVslit' in kwargs['scanType']: 
        list_position=[GRT,'S1H','S1V','S2H','S2v']
        if mono_grating_get('HEG'): 
            slit(300)
            c=2
        else: 
            slit(200)
            c=1

        diagnostics_all_out(diode_stay_in=True)
        print("\n\n================== Mono/slit scans:")     
        if kwargs['monoVslit_quick']:
            for i,slit_name in enumerate(['slit2B','slit1A'])
                print('\n---------- Scanning '+slit_name+':\n') 
                flag=2
                while flag==2:
                    #scanning
                    monoVslit_quick_scan(slit_name,**kwargs)

                    #plotting
                    try:
                        scanNum_v = last_mda()-1
                        scanNum_h = last_mda()
                        V2 = monoVslit_quick_plot(scanNum_v,detDiode,c)
                        H2 = monoVslit_quick_plot(scanNum_h,detDiode,1)

                    except:
                        print('Unable to fit position; try to adjust xrange.')

                    flag=interactive_fct(mirror=True)    
                    #want to refit       
                    while flag == 3:
                        fitV2=input('do you want to FIT mda #'+str(m-1)+' ie slit2-V (y or n)? >')
                        if fitV2.lower() == 'y' or fitV2.lower() ==  'yes': 
                            x1=input('x1 >')
                            x2=input('x2 >')
                            xrange=[float(x1),float(x2)]
                            V2 = monoVslit_quick_plot(scanNum_v,detDiode,c,xrange=xrange)
                            foo=input('do you want to replace '+slit_name+'-V value in list_position (y or n)? >')
                            if foo.lower() == 'y' or foo.lower() ==  'yes':
                                list_position[1]=V2
                        
                        fitH2=input('do you want to FIT mda #'+str(m)+' ie slit2-H (y or n)? >')
                        if fitH2.lower() == 'y' or fitH2.lower() ==  'yes': 
                            x1=input('x1 >')
                            x2=input('x2 >')
                            xrange=[float(x1),float(x2)]
                            H2 = monoVslit_quick_plot(scanNum_h,detDiode,1,xrange=xrange)
                            foo=input('do you want to REPLACE '+slit_name+'-H value in list_position (y or n)? >')
                            if foo.lower() == 'y' or foo.lower() ==  'yes':
                                list_position[1]=H2
                        
                    if slit_name = 'slit2B':
                        M0M1_steering(H2,V2)
                        list_position[4] = V2
                        list_position[3] = H2
                        mirror = True
                    elif slit_name: 'slit1A':
                        list_position[2] = V2
                        list_position[1] = H2
                        mirror = False

                    foo=input('\nare you happy with the fit (y or n)? >')
                    if foo.lower() == 'y' or foo.lower() ==  'yes': 
                        flag=interactive_fct(mirror=mirror) 
                        
                    #happy with fitting but want to adjust roll
                    if flag == 4:
                        x=input('New M1 roll value >')
                        x=float(x)
                        try:
                            FMB_mirror_move(1,'RZ',x)
                            sleep(2);
                            FMB_mirror_get(1)
                            print('\nrepeating scan...\n')
                            flag=2
                        except:
                            print('\ninvalid input; unable to move M1...\n')
                            flag=interactive_fct(mirror=True) 
                
            print('\nBeam center fit @ '+str(list_position))
            foo=input('do you want to update the slit dictionary (y or n)? >')
            if foo.lower() == 'y' or foo.lower() ==  'yes':
                slit_list=input('do you want to use '+str(list_position)+' (y or n)? >')
                if foo.lower() == 'y' or foo.lower() ==  'yes':
                    update_slit_dict(list_position)
                else:
                    update_slit_dict()

        else:    # 2D map
            mvID(500)
            flag=2
            while flag==2:
                beam_profile(GRT,["2V","2H","1V","1H"])

                #interactive
                flag=interactive_fct()
                    
    ###### Check flux: 
    if 'flux' in kwargs['scanType']:
        diode_c('In')
        print("\n\n================== Check Flux:")
        flag=2
        while flag==2:
            check_flux_scan(stay=True)

            scanNum = last_mda()
            try:
                plot_mda(scanNum,detDiode,Flux=3);plt.show()
            except:
                print('Unable to plot. Try:') 
                print("plot_mda("+str(m)+","+str(detDiode)+",flux=3);plt.show()")  
            flag=interactive_fct()

    diagnostics_all_out()

    print("\n\n================== All done:")
    print("\nDon't forget to put back the user folder !!!!")
    

    ###### The end:
    if list_position:
        print('\nBeam center fit @ ['+GRT+',2V,2H,1V,1H]:',list_position)
    if kwargs['sound']:
        playsound()
        
    return list_position
    



def update_slit_dict(mylist=None,**kwargs):
    """
    Interactive function to update the slit position dictionary (Dict_Slit.txt)
    mylist=['GRT',2V,2H,1V,1H]
    **kwargs
        readOnly == False, if true just pring the current slit dictionary
    """
    filepath = "/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/"
    filename ='Dict_Slit.txt'
    
    kwargs.setdefault("readOnly",False)

    # print('\n\nWARNING: 2V slit should always be centered at zero ie steer M1 center beam on the grating')
    # print('Increasing the roll moves the beam more positive: +0.05 pitch => ~ +0.04 slit position; Use:\n')
    # print("Get_Mirror(1);Move_M1('RZ',x)")
    # print("energy(500);mono(505);slit(200)")
    # print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.25, -2, 2, 0.25]) #for MEG; slit_parameters=[0.5, -2, 2, 0.25] for HEG")
    # print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.50, -2, 2, 0.25]) #for HEG")
    # print('Once steering is completed, check 2H position:')
    # print("Scan_NarrowSlit_Go(which='2H',slit_parameters=[0.25, -6, 6, 0.25]) ")
    

    try:
        slit_position=read_dict(filename)
        print('\nCurrent dictionary:\n')
        print(slit_position)

    except KeyError:
        print("Unable to read dictionary")
        return
    
    if kwargs['readOnly']== True:
        return slit_position
        
    else:
        if mylist:
            if len(mylist) == 5:
                grt,s2v,s2h,s1v,s1h=mylist
            else:
                print('\nInvalid list. No changes made in file.')
                return              

        else:
            grt=input('\nWhich grating do you want to update (HEG or MEG) >')
            s2v=input('New Slit 2B-V center >')
            s2h=input('New Slit 2B-H center >')
            s1v=input('New Slit 1A-V center >')
            s1h=input('New Slit 1A-H center >')

            if grt == '' or s1h == '' or s1v == '' or s2h == '' or s2v == '':
                print('\nMissing input. No changes made in file.')
                return

            if grt[0] == '"' or grt[0]=="'": grt = grt[1:4]
            
        #    new_slit_position={grt.upper():{'S1H':float(s1h),'S1V':float(s1v),'S2H':float(s2h),'S2V':float(s2v)}}   # change order to match scan/fit order
        if grt.upper() == 'MEG' or grt.upper() == 'HEG':
            new_slit_position={grt.upper():{'S2V':float(s2v),'S2H':float(s2h),'S1V':float(s1v),'S1H':float(s1h)}}
        else:
            print('\nInvalid grating name. No changes made in file.')
            return            


        slit_position.update(new_slit_position)


        with open(join(filepath, filename), "a+") as f:
            f.write('\n======= '+today()+': \n')
            f.write(str(slit_position))
            f.write('\n')
            print('\nWriting dictionary to:',join(filepath, filename))
        sleep(2)
        apertures_set()
        return slit_position


    
def update_slit_dict(**kwargs):
    """
    Interactive function to update the slit position dictionary (Dict_Slit.txt)
    
    **kwargs
        readOnly == False, if true just pring the current slit dictionary
    """
    filepath = "/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/"
    filename ='Dict_Slit.txt'
    
    kwargs.setdefault("readOnly",False)

    print('\n\nWARNING: 2V slit should always be centered at zero ie steer M1 center beam on the grating')
    print('Increasing the roll moves the beam more positive: +0.05 pitch => ~ +0.04 slit position; Use:\n')
    print("Get_Mirror(1);Move_M1('RZ',x)")
    print("energy(500);mono(505);slit(200)")
    print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.25, -2, 2, 0.25]) #for MEG; slit_parameters=[0.5, -2, 2, 0.25] for HEG")
    print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.50, -2, 2, 0.25]) #for HEG")
    print('Once steering is completed, check 2H position:')
    print("Scan_NarrowSlit_Go(which='2H',slit_parameters=[0.25, -6, 6, 0.25]) ")
    
    try:
        slit_position=read_dict(filename)
        print('\nCurrent dictionary:\n')
        print(slit_position)

    except KeyError:
        print("Unable to read previous dictionary")
        return
    
    if kwargs['readOnly']== True:
        return slit_position
        
    else:
        grt=input('\nWhich grating do you want to update (HEG or MEG) >')
        s2v=input('New Slit 2B-V center >')
        s2h=input('New Slit 2B-H center >')
        s1v=input('New Slit 1A-V center >')
        s1h=input('New Slit 1A-H center >')

        if grt == '' or s1h == '' or s1v == '' or s2h == '' or s2v == '':
            print('\nMissing input. No changes made in file.')
            return

        if grt[0] == '"' or grt[0]=="'": grt = grt[1:4]

    #    new_slit_position={grt.upper():{'S1H':float(s1h),'S1V':float(s1v),'S2H':float(s2h),'S2V':float(s2v)}}   # change order to match scan/fit order
        new_slit_position={grt.upper():{'S2V':float(s2v),'S2H':float(s2h),'S1V':float(s1v),'S1H':float(s1h)}}
        slit_position.update(new_slit_position)


        with open(join(filepath, filename), "a+") as f:
            f.write('\n======= '+today()+': \n')
            f.write(str(slit_position))
            f.write('\n')
            print('\nWriting dictionary to:',join(filepath, filename))

        apertures_set()
        return slit_position

